@@ -97,9 +97,31 @@ static unsigned int get_hash_value(hash_obj* self, char* key)
    }
    return i % self->mTableSize;
 }
-   
+
+static void resize(hash_obj* self) 
+{
+    self->mTableSize *= 5;
+    self->mTable = (hash_it**)MALLOC(sizeof(hash_it*) * self->mTableSize);
+    memset(self->mTable, 0, sizeof(hash_it*)*self->mTableSize);
+
+    hash_it* it = self->mEntryIt;
+    while(it) {
+        unsigned int hash_key = get_hash_value(self, it->mKey);
+
+        it->mCollisionIt = self->mTable[hash_key];
+        
+        self->mTable[hash_key] = it;
+
+        it = it->mNextIt;
+    }
+}
+
 void hash_put(hash_obj* self, char* key, void* item)
 {
+    if(self->mCounter >= self->mTableSize) {
+        resize(self);
+    }
+    
     unsigned int hash_key = get_hash_value(self, key);
     
     hash_it* it = self->mTable[ hash_key ];
@@ -116,6 +138,7 @@ void hash_put(hash_obj* self, char* key, void* item)
     
     self->mTable[hash_key] = new_it;
     self->mEntryIt = new_it;
+    self->mCounter++;
 }
 
 ///////////////////////////////////////////////////////////////////////
@@ -154,6 +177,8 @@ void hash_erase(hash_obj* self, char* key)
       erase_from_list(self, it);
       
       hash_it_release(it);
+
+      self->mCounter--;
    }
    else {
       hash_it* prev_it = it;
@@ -170,6 +195,7 @@ void hash_erase(hash_obj* self, char* key)
          prev_it = it;
          it = it->mCollisionIt;
       }
+      self->mCounter--;
    }
 }
 
@@ -191,6 +217,7 @@ void hash_clear(hash_obj* self)
    memset(self->mTable, 0, sizeof(hash_it*)*self->mTableSize);
    
    self->mEntryIt = NULL;
+   self->mCounter = 0;
 }
 
 ///////////////////////////////////////////////////////////////////////
@@ -217,27 +244,36 @@ void hash_replace(hash_obj* self, char* key, void* item)
 ///////////////////////////////////////////////////////////////////////
 void hash_show(hash_obj* self, char* fname)
 {
-   char tmp[4096];
+   char tmp[8096];
    int i;
    int max;
    hash_it* it;
-   xstrncpy(tmp, "", 4096);
+   xstrncpy(tmp, "", 8096);
    
    max = self->mTableSize;
    for(i=0; i<max; i++) {
-      snprintf(tmp + strlen(tmp), 4096-strlen(tmp), "table[%d]: ", i);
+      snprintf(tmp + strlen(tmp), 8096-strlen(tmp), "table[%d]: ", i);
       
       it = self->mTable[i];
       while(it) {
-         snprintf(tmp + strlen(tmp), 4096-strlen(tmp), "item[\"%s\"], ", it->mKey);
+         snprintf(tmp + strlen(tmp), 8096-strlen(tmp), "item[\"%s\"], ", it->mKey);
          it = it->mCollisionIt;
       }
     
-    xstrncat(tmp, "\n", 4096);
+    xstrncat(tmp, "\n", 8096);
    }
+
     
-    FILE* f = fopen(fname, "w");
+    FILE* f = fopen(fname, "a");
+    fprintf(f, "-------------------------------------\n");
     fprintf(f, "%s", tmp);
+    fprintf(f, "-------------------------------------\n");
+    it = self->mEntryIt;
+    while(it) {
+        fprintf(f, "%s\n", it->mKey);
+        it = it->mNextIt;
+    }
+    fprintf(f, "-------------------------------------\n");
     fclose(f);
 }
 
@@ -290,15 +326,7 @@ char* hash_key(hash_obj* self, void* item)
 ///////////////////////////////////////////////////////////////////////
 int hash_count(hash_obj* self)
 {
-   int result = 0;
-   hash_it* it = self->mEntryIt;
-
-   while(it) {
-      result++;
-      it = it->mNextIt;
-   }
-               
-   return result;
+   return self->mCounter;
 }   
 
 hash_it* hash_loop_begin(hash_obj* self) { return self->mEntryIt; }
