@@ -503,7 +503,7 @@ static BOOL statment_tree_external_command(sStatment* statment, vector_obj* argv
 }
 
 /// argvを作る
-static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, BOOL* return_, sRFd* nextin, sRFd* pipein, char* sname, int sline, vector_obj* envs, vector_obj* psubs, sObject* object, sFunction* running_fun, sClass* running_class)
+static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, BOOL* return_, BOOL enable_return, BOOL* break_, BOOL enable_break, sRFd* nextin, sRFd* pipein, char* sname, int sline, vector_obj* envs, vector_obj* psubs, sObject* object, sFunction* running_fun, sClass* running_class)
 {
     int ii;
     for(ii=0; ii<vector_size(command->mArgs); ii++) {
@@ -519,15 +519,12 @@ static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_
                 sStatments* statments = arg->mBody;
                 sWFd* pipeout = sWFd_new(-1);
                 int rcode = run(statments, "quick command expansion"
-                    , pipeout, pipein, 2, return_, parent_blocks, FALSE
+                    , pipeout, pipein, 2, return_, enable_return, break_, enable_break, parent_blocks, FALSE
                     , envs, object, running_fun, running_class);
 
                 if(rcode == -1) {
                     return FALSE;
                 }
-                if(*return_) {
-                    return TRUE;;
-                }
 
                 vector_add(argv, STRING_NEW(pipeout->mBuffer));
                 }
@@ -538,15 +535,12 @@ static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_
                 sStatments* statments = arg->mBody;
                 sWFd* pipeout = sWFd_new(-1);
                 int rcode = run(statments, "quick command expansion"
-                    , pipeout, pipein, 2, return_, parent_blocks, FALSE
+                    , pipeout, pipein, 2, return_, enable_return, break_, enable_break, parent_blocks, FALSE
                     , envs, object, running_fun, running_class);
 
                 if(rcode == -1) {
                     return FALSE;
                 }
-                if(*return_) {
-                    return TRUE;
-                }
 
                 sRFd* tmp = sRFd_new2(-1, pipeout->mBuffer);
                 while(1) {
@@ -578,15 +572,12 @@ static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_
                 sStatments* statments = arg->mBody;
                 sWFd* pipeout = sWFd_new(-1);
                 int rcode = run(statments, "process substitution"
-                    , pipeout, pipein, 2, return_, parent_blocks, FALSE
+                    , pipeout, pipein, 2, return_, enable_return, break_, enable_break, parent_blocks, FALSE
                     , envs, object, running_fun, running_class);
 
                 if(rcode == -1) {
                     return FALSE;
                 }
-                if(*return_) {
-                    return TRUE;
-                }
 
                 /// 一時ファイルを作る ///
                 char fname[PATH_MAX];
@@ -689,7 +680,7 @@ static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_
 // 文を回す
 // TRUE 正常終了
 // FALSE エラー
-static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipeout, sRFd* pipein, int pipeerr, char* title, int* rcode, BOOL* return_, vector_obj* parent_blocks, char* sname, int sline, vector_obj* envs, vector_obj* psubs, sObject* object, sFunction* running_fun, sClass* running_class) 
+static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipeout, sRFd* pipein, int pipeerr, char* title, int* rcode, BOOL* return_, BOOL enable_return, BOOL* break_, BOOL enable_break, vector_obj* parent_blocks, char* sname, int sline, vector_obj* envs, vector_obj* psubs, sObject* object, sFunction* running_fun, sClass* running_class) 
 {
     sRFd* nextin;
     int nexterr = pipeerr;
@@ -752,7 +743,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
         // argvを作成 sArgから
         vector_obj* argv = VECTOR_NEW(30);
         vector_obj* blocks = VECTOR_NEW(30);
-        if(!statment_tree_make_argv(command, argv, blocks, parent_blocks, return_, nextin, pipein, sname, sline, envs, psubs, object, running_fun, running_class)) {
+        if(!statment_tree_make_argv(command, argv, blocks, parent_blocks, return_, enable_return, break_, enable_break, nextin, pipein, sname, sline, envs, psubs, object, running_fun, running_class)) {
             return FALSE;
         }
         if(*return_) {
@@ -864,8 +855,9 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
 
                 *rcode = run(klass->statments, title, nextout
                                 , nextin, nexterr
-                                , return_, parent_blocks, FALSE, envs, object2, NULL, klass);
+                                , return_, TRUE, break_, FALSE, parent_blocks, FALSE, envs, object2, NULL, klass);
                 *return_ = FALSE; // ユーザー関数が復帰したらreturn終わり
+                *break_ = FALSE;
 
                 // スタックを埋める
                 vector_pop_back(gStackFrame);
@@ -937,8 +929,9 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 
                 *rcode = run(klass->statments, title, nextout
                                 , nextin, nexterr
-                                , return_, parent_blocks, FALSE, envs, object2, NULL, klass);
+                                , return_, TRUE, break_, FALSE, parent_blocks, FALSE, envs, object2, NULL, klass);
                 *return_ = FALSE; // ユーザー関数が復帰したらreturn終わり
+                *break_ = FALSE;
 
                 // スタックを埋める
                 vector_pop_back(gStackFrame);
@@ -981,8 +974,9 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 
                 *rcode = run(statments, title, nextout
                                 , nextin, nexterr
-                                , return_, parent_blocks, FALSE, envs, object2, NULL, NULL);
+                                , return_, TRUE, break_, FALSE, parent_blocks, FALSE, envs, object2, NULL, NULL);
                 *return_ = FALSE; // ユーザー関数が復帰したらreturn終わり
+                *break_ = FALSE;
 
                 // スタックを埋める
                 vector_pop_back(gStackFrame);
@@ -1067,8 +1061,17 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
 
             /// 実行 ///
             *rcode = run(method->statments, title
-                , nextout, nextin, nexterr, return_, blocks, FALSE, envs, object2, method, running_class);
-            *return_ = FALSE; // ユーザー関数が復帰したらreturn終わり
+                , nextout, nextin, nexterr, return_, TRUE, break_, FALSE, blocks, FALSE, envs, object2, method, running_class);
+
+            *break_ = FALSE;
+            if(strcmp(string_c_str(method->arg_name), "") == 0) {
+                *return_ = FALSE; // ユーザー関数が復帰したらreturn終わり
+            }
+            else {
+                if(enable_return && *return_) {
+                    return TRUE;
+                }
+            }
 
             /// スタックフレームの記録を消す
             string_obj* sname_strace = vector_pop_back(gStackTraceFName);
@@ -1170,8 +1173,16 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
 
             /// 実行 ///
             *rcode = run(fun->statments, title
-                , nextout, nextin, nexterr, return_, blocks, FALSE, envs, object, fun, running_class);
-            *return_ = FALSE; // ユーザー関数が復帰したらreturn終わり
+                , nextout, nextin, nexterr, return_, TRUE, break_, FALSE, blocks, FALSE, envs, object, fun, running_class);
+            *break_ = FALSE;
+            if(strcmp(string_c_str(fun->arg_name), "") == 0) {
+                *return_ = FALSE; // ユーザー関数が復帰したらreturn終わり
+            }
+            else {
+                if(enable_return && *return_) {
+                    return TRUE;
+                }
+            }
 
             /// スタックフレームの記録を消す
             string_obj* sname_strace = vector_pop_back(gStackTraceFName);
@@ -1285,7 +1296,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
             }
 
             BOOL r = statment_tree_internal_commands(command, rcode
-                , argv, blocks, parent_blocks, nextout, nextin, nexterr, j, title, !first_program || exist_globalin || exist_stdinpipe,return_, sname, sline, envs, object, running_fun, running_class);
+                , argv, blocks, parent_blocks, nextout, nextin, nexterr, j, title, !first_program || exist_globalin || exist_stdinpipe, return_, enable_return, break_, enable_break, sname, sline, envs, object, running_fun, running_class);
 
             /// 終了コード ///
             if(statment->mRCodeReverse) {
@@ -1301,11 +1312,6 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
             if(!r) {
                 return FALSE;
             }
-
-            /// return
-            if(*return_) {
-                return TRUE;
-            }
         }
 
         /// 外部コマンド ///
@@ -1393,6 +1399,14 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 nextin = sRFd_new(-1);
             }
         }
+
+        /// 大域脱出
+        if(enable_return && *return_) {
+            return TRUE;
+        }
+        if(enable_break && *break_) {
+            return TRUE;
+        }
     }
 
     return TRUE;
@@ -1518,7 +1532,7 @@ void run_wait_cprogs(sStatment* statment, int* rcode, sWFd* pipeout, int last_pr
 }
 
 // 複文を実行したい関数。本体。
-int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pipeerr, BOOL* return_, vector_obj* parent_blocks, BOOL try_, vector_obj* envs, sObject* object, sFunction* running_fun, sClass* running_class)
+int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pipeerr, BOOL* return_, BOOL enable_return, BOOL* break_, BOOL enable_break, vector_obj* parent_blocks, BOOL try_, vector_obj* envs, sObject* object, sFunction* running_fun, sClass* running_class)
 {
     int rcode = 0;
 
@@ -1538,10 +1552,7 @@ int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pip
                 if(parse(string_c_str(cmdline), sname, &sline, estatments)) 
                 {
                     rcode = run(estatments, title, pipeout, pipein
-                        , pipeerr, return_, parent_blocks, FALSE, envs, object, running_fun, running_class);
-                    if(*return_) {
-                        return 0;
-                    }
+                        , pipeerr, return_, enable_return, break_, enable_break, parent_blocks, FALSE, envs, object, running_fun, running_class);
 
                     if(rcode < 0) {
                         return rcode;
@@ -1565,7 +1576,7 @@ int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pip
                           , &last_program_pid
                           , pipeout, pipein, pipeerr
                           , title, &rcode
-                          , return_, parent_blocks , sname, sline
+                          , return_, enable_return, break_, enable_break, parent_blocks , sname, sline
                           , envs, psubs, object, running_fun, running_class);
 
             /// プロセス回収 ///
@@ -1591,7 +1602,7 @@ int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pip
 
                 sRFd* nextin2 = sRFd_new(f);
                 int rcode = run(psub->mStatments, "process substitution2"
-                    , pipeout, nextin2, 2, return_, parent_blocks, FALSE
+                    , pipeout, nextin2, 2, return_, enable_return, break_, enable_break, parent_blocks, FALSE
                     , envs, object, running_fun, running_class);
 
                 (void)close(f);
@@ -1599,9 +1610,6 @@ int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pip
                 if(rcode < 0) {
                     return rcode;
                 }
-                if(*return_) {
-                    return 0;
-                }
             }
         }
 
@@ -1616,9 +1624,6 @@ int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pip
             gKitutukiSigInt = FALSE;
             return -1;
         }
-        else if(rcode == SIGUSR2) {
-            return rcode;
-        }
         else if(statment->mTerminated == kTOrOr && rcode == 0) {
             while(i<vector_size(statments->mStatments) 
                     && statment->mTerminated != kTNormal) 
@@ -1638,6 +1643,14 @@ int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pip
         else if(try_ && rcode != 0) {
             return -1;
         }
+
+        /// 大域脱出
+        if(enable_return && *return_) {
+            return TRUE;
+        }
+        if(enable_break && *break_) {
+            return TRUE;
+        }
     }
 
     /// 空文の場合 ///
@@ -1719,10 +1732,11 @@ int saphire_shell(char* command, char* fname, sWFd* pipeout, sRFd* pipein, int p
     else {
         /// 実行 ///
         int return_ = FALSE;
+        int break_ = FALSE;
         vector_obj* envs = VECTOR_NEW(10);
         int rcode = run(statments, string_c_str(fname2)
                     , pipeout, pipein, pipeerr
-                    , &return_, NULL, FALSE, envs, NULL, NULL, NULL);
+                    , &return_, TRUE, &break_, FALSE, NULL, FALSE, envs, NULL, NULL, NULL);
 
         /// 解放 ///
         return rcode;
@@ -1776,10 +1790,11 @@ int saphire_shell4(char* command, char* fname, sWFd* pipeout, sRFd* pipein, int
     else {
         /// 実行 ///
         int return_ = FALSE;
+        int break_ = FALSE;
         vector_obj* envs = VECTOR_NEW(10);
         int rcode = run(statments, string_c_str(fname2)
                     , pipeout, pipein, pipeerr
-                    , &return_, NULL, FALSE, envs, object, NULL, NULL);
+                    , &return_, TRUE, &break_, FALSE, NULL, FALSE, envs, object, NULL, NULL);
 
         /// 解放 ///
         return rcode;
@@ -2015,9 +2030,10 @@ BOOL saphire_compile2(char* cmdline, char* fname, sStatments* statments)
 int saphire_run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pipeerr)
 {
     int return_ = FALSE;
+    int break_ = FALSE;
     vector_obj* envs = VECTOR_NEW(10);
     int r = run(statments, title, pipeout, pipein, pipeerr
-            , &return_, NULL, FALSE, envs, NULL, NULL, NULL);
+            , &return_, TRUE, &break_, FALSE, NULL, FALSE, envs, NULL, NULL, NULL);
 
     return r;
 }
